@@ -7,7 +7,9 @@ from typing import List, Dict
 from collections import namedtuple
 
 import ascend.protos.api.api_pb2 as api_pb2
+import ascend.protos.ascend.ascend_pb2 as ascend_pb2
 import ascend.protos.component.component_pb2 as component_pb2
+import ascend.protos.connection.connection_pb2 as connection_pb2
 import ascend.protos.core.core_pb2 as core_pb2
 import ascend.protos.io.io_pb2 as io_pb2
 import ascend.protos.operator.operator_pb2 as operator_pb2
@@ -326,15 +328,16 @@ class DataFeed(Definition):
       proto.pubToRoles = ""
     else:
       proto.open = False
-      share_roles = []
+      unique_role_ids = set([])
       try:
         for ds in self.data_services_shared_with:
-          share_roles.append(ds_to_role_map[ds])
+          if ds in ds_to_role_map:
+            unique_role_ids.add(ds_to_role_map[ds])
         if not self.hidden_from_host_data_service:
-          share_roles.append(ds_to_role_map[host_data_service_id])
+          unique_role_ids.add(ds_to_role_map[host_data_service_id])
       except KeyError as e:
         raise KeyError(f"missing role for data service {e}")
-      proto.pubToRoles = ','.join(share_roles)
+      proto.pubToRoles = ','.join(unique_role_ids)
 
     return proto
 
@@ -460,12 +463,87 @@ class Dataflow(Definition):
     return api_pb2.Dataflow(id=self.id, name=self.name, description=self.description)
 
 
+class Credential(Definition):
+  """A credential"""
+  def __init__(self, id: str, name: str, credential: io_pb2.Credentials):
+    """
+    Create a Credential definition.
+
+    Parameters:
+    - id: a uuid that is auto-generated by Ascend
+    - name: user-friendly name
+    - credential: the credential values
+    Returns: a new Credential instance.
+    """
+    self.id = id
+    self.name = name
+    self.credential = credential
+
+  def to_proto(self):
+    try:
+      return api_pb2.Credentials(credential_id=self.id, name=self.name, credential=self.credential)
+    except KeyError as e:
+      raise KeyError(f"unresolved id for credential {e}")
+
+  @staticmethod
+  def from_proto(proto: api_pb2.Credentials) -> 'Credential':
+    try:
+      return Credential(id=proto.credential_id, name=proto.name, credential=proto.credential)
+    except KeyError as e:
+      raise KeyError(f"unresolved id for credential {e}")
+
+
+class Connection(Definition):
+  """A connection"""
+  def __init__(self, id: str, name: str, type_id: str, credential_id: str, details: Dict[str, ascend_pb2.Value]):
+    """
+    Create a Connection definition.
+
+    Parameters:
+    - id: a uuid that is auto-generated by Ascend
+    - name: user-friendly name
+    - type_id: the type of connection
+    - credential_id: the id of the credential to use
+    - details: a dictionary of the connection details
+    Returns: a new Connection instance.
+    """
+    self.id = id
+    self.name = name
+    self.type_id = type_id
+    self.credential_id = credential_id
+    self.details = details
+
+  def to_proto(self):
+    try:
+      return io_pb2.Connection(id=connection_pb2.Id(value=self.id),
+                               name=self.name,
+                               type_id=connection_pb2.Type.Id(value=self.type_id),
+                               credential_id=io_pb2.Credentials.Id(value=self.credential_id) if self.credential_id else None,
+                               details=self.details)
+    except KeyError as e:
+      raise KeyError(f"unresolved id for connection {e}")
+
+  @staticmethod
+  def from_proto(proto: io_pb2.Connection) -> 'Connection':
+    try:
+      details = {k: proto.details[k] for k in proto.details.keys()} if hasattr(proto, "details") else None
+      return Connection(id=proto.id.value, name=proto.name, type_id=proto.type_id.value, credential_id=proto.credential_id.value, details=details)
+    except KeyError as e:
+      raise KeyError(f"unresolved id for component {e}")
+
+
 class DataService(Definition):
   """A DataService is the highest-level organizational structure in Ascend, and the primary
   means of controlling access. It contains one or more Dataflows and can communicate with
   other Data Services via Data Feeds.
   """
-  def __init__(self, id: str, name: str, description: str = None, dataflows: List[Dataflow] = []):
+  def __init__(self,
+               id: str,
+               name: str,
+               description: str = None,
+               credentials: List[Credential] = [],
+               connections: List[Connection] = [],
+               dataflows: List[Dataflow] = []):
     """
     Create a DataService definition.
 
@@ -479,6 +557,8 @@ class DataService(Definition):
     self.id = id
     self.name = name
     self.description = description
+    self.credentials = credentials
+    self.connections = connections
     self.dataflows = dataflows
 
   def to_proto(self):
