@@ -44,7 +44,7 @@ class InlineCode:
     if self.base64_encoded:
       return f'''base64.b64encode(pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), {rel_path_components})).read_bytes()).decode()'''
     else:
-      return f'''pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), {rel_path_components})).read_text()'''
+      return f'''pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), {rel_path_components})).read_text(encoding="utf-8")'''
 
 
 def download_dataflow(client: Client, data_service_id: str, dataflow_id: str, resource_base_path: str = "."):
@@ -76,10 +76,10 @@ def download_dataflow(client: Client, data_service_id: str, dataflow_id: str, re
 
   rendered_dataflow, inline_code_list = render_dataflow(data_service_id, dataflow, client.hostname, dataflow_resource_path, resource_base_path)
   for inline_code in inline_code_list:
-    with open(inline_code.resource_path, "w") as f:
+    with open(inline_code.resource_path, "w", encoding="utf-8") as f:
       f.write(inline_code.code)
 
-  with open(dataflow_path, "w") as f:
+  with open(dataflow_path, "w", encoding="utf-8") as f:
     f.write(rendered_dataflow)
 
 
@@ -115,10 +115,10 @@ def write_data_service(hostname: str, data_service: DataService, resource_base_p
 
   rendered_data_service, inline_code_list = render_data_service(data_service, hostname, resource_base_path)
   for inline_code in inline_code_list:
-    with open(inline_code.resource_path, "w") as f:
+    with open(inline_code.resource_path, "w", encoding="utf-8") as f:
       f.write(inline_code.code)
 
-  with open(data_service_path, "w") as f:
+  with open(data_service_path, "w", encoding="utf-8") as f:
     f.write(rendered_data_service)
 
 
